@@ -1,4 +1,4 @@
-/* $OpenBSD: ts.c,v 1.17 2021/12/12 20:22:59 tb Exp $ */
+/* $OpenBSD: ts.c,v 1.24 2022/09/11 18:08:17 tb Exp $ */
 /* Written by Zoltan Glozik (zglozik@stones.com) for the OpenSSL
  * project 2002.
  */
@@ -80,74 +80,302 @@ static ASN1_OBJECT *txt2obj(const char *oid);
 static CONF *load_config_file(const char *configfile);
 
 /* Query related functions. */
-static int query_command(const char *data, char *digest,
-    const EVP_MD * md, const char *policy, int no_nonce,
-    int cert, const char *in, const char *out, int text);
+static int query_command(const char *data, char *digest, const EVP_MD *md,
+    const char *policy, int no_nonce, int cert, const char *in, const char *out,
+    int text);
 static BIO *BIO_open_with_default(const char *file, const char *mode,
-    FILE * default_fp);
-static TS_REQ *create_query(BIO * data_bio, char *digest, const EVP_MD * md,
+    FILE *default_fp);
+static TS_REQ *create_query(BIO *data_bio, char *digest, const EVP_MD *md,
     const char *policy, int no_nonce, int cert);
-static int create_digest(BIO * input, char *digest,
-    const EVP_MD * md, unsigned char **md_value);
+static int create_digest(BIO *input, char *digest, const EVP_MD *md,
+    unsigned char **md_value);
 static ASN1_INTEGER *create_nonce(int bits);
 
 /* Reply related functions. */
-static int reply_command(CONF * conf, char *section,
-    char *queryfile, char *passin, char *inkey,
-    char *signer, char *chain, const char *policy,
-    char *in, int token_in, char *out, int token_out,
-    int text);
-static TS_RESP *read_PKCS7(BIO * in_bio);
-static TS_RESP *create_response(CONF * conf, const char *section,
-    char *queryfile, char *passin, char *inkey,
-    char *signer, char *chain, const char *policy);
-static ASN1_INTEGER *serial_cb(TS_RESP_CTX * ctx, void *data);
+static int reply_command(CONF *conf, char *section, char *queryfile,
+    char *passin, char *inkey, char *signer, char *chain, const char *policy,
+    char *in, int token_in, char *out, int token_out, int text);
+static TS_RESP *read_PKCS7(BIO *in_bio);
+static TS_RESP *create_response(CONF *conf, const char *section,
+    char *queryfile, char *passin, char *inkey, char *signer, char *chain,
+    const char *policy);
+static ASN1_INTEGER *serial_cb(TS_RESP_CTX *ctx, void *data);
 static ASN1_INTEGER *next_serial(const char *serialfile);
-static int save_ts_serial(const char *serialfile, ASN1_INTEGER * serial);
+static int save_ts_serial(const char *serialfile, ASN1_INTEGER *serial);
 
 /* Verify related functions. */
-static int verify_command(char *data, char *digest, char *queryfile,
-    char *in, int token_in,
-    char *ca_path, char *ca_file, char *untrusted);
+static int verify_command(char *data, char *digest, char *queryfile, char *in,
+    int token_in, char *ca_path, char *ca_file, char *untrusted);
 static TS_VERIFY_CTX *create_verify_ctx(char *data, char *digest,
-    char *queryfile,
-    char *ca_path, char *ca_file,
-    char *untrusted);
+    char *queryfile, char *ca_path, char *ca_file, char *untrusted);
 static X509_STORE *create_cert_store(char *ca_path, char *ca_file);
-static int verify_cb(int ok, X509_STORE_CTX * ctx);
+static int verify_cb(int ok, X509_STORE_CTX *ctx);
+
+enum mode {
+	CMD_NONE, CMD_QUERY, CMD_REPLY, CMD_VERIFY
+};
+
+static struct {
+	char *ca_file;
+	char *ca_path;
+	int cert;
+	char *chain;
+	char *configfile;
+	char *data;
+	char *digest;
+	char *in;
+	char *inkey;
+	const EVP_MD *md;
+	int mode;
+	int no_nonce;
+	char *out;
+	char *passin;
+	char *policy;
+	char *queryfile;
+	char *section;
+	char *signer;
+	int text;
+	int token_in;
+	int token_out;
+	char *untrusted;
+} ts_config;
+
+static int
+ts_opt_md(int argc, char **argv, int *argsused)
+{
+	char *name = argv[0];
+
+	if (*name++ != '-')
+		return (1);
+
+	if ((ts_config.md = EVP_get_digestbyname(name)) == NULL)
+		return (1);
+
+	*argsused = 1;
+	return (0);
+}
+
+static int
+ts_opt_query(void)
+{
+	if (ts_config.mode != CMD_NONE)
+		return (1);
+	ts_config.mode = CMD_QUERY;
+	return (0);
+}
+
+static int
+ts_opt_reply(void)
+{
+	if (ts_config.mode != CMD_NONE)
+		return (1);
+	ts_config.mode = CMD_REPLY;
+	return (0);
+}
+
+static int
+ts_opt_verify(void)
+{
+	if (ts_config.mode != CMD_NONE)
+		return (1);
+	ts_config.mode = CMD_VERIFY;
+	return (0);
+}
+
+static const struct option ts_options[] = {
+	{
+		.name = "CAfile",
+		.argname = "file",
+		.desc = "Certificate Authority file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.ca_file,
+	},
+	{
+		.name = "CApath",
+		.argname = "path",
+		.desc = "Certificate Authority path",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.ca_path,
+	},
+	{
+		.name = "cert",
+		.desc = "Include signing certificate in the response",
+		.type = OPTION_FLAG,
+		.opt.flag = &ts_config.cert,
+	},
+	{
+		.name = "chain",
+		.argname = "file",
+		.desc = "PEM certificates that will be included in the response",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.chain,
+	},
+	{
+		.name = "config",
+		.argname = "file",
+		.desc = "Specify an alternative configuration file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.configfile,
+	},
+	{
+		.name = "data",
+		.argname = "file",
+		.desc = "Data file for which the time stamp request needs to be created",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.data,
+	},
+	{
+		.name = "digest",
+		.argname = "arg",
+		.desc = "Specify the message imprint explicitly without the data file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.digest,
+	},
+	{
+		.name = "in",
+		.argname = "file",
+		.desc = "Input file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.in,
+	},
+	{
+		.name = "inkey",
+		.argname = "file",
+		.desc = "Input key file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.inkey,
+	},
+	{
+		.name = "no_nonce",
+		.desc = "Specify no nonce in the request",
+		.type = OPTION_FLAG,
+		.opt.flag = &ts_config.no_nonce,
+	},
+	{
+		.name = "out",
+		.argname = "file",
+		.desc = "Output file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.out,
+	},
+	{
+		.name = "passin",
+		.argname = "src",
+		.desc = "Private key password source",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.passin,
+	},
+	{
+		.name = "policy",
+		.argname = "object_id",
+		.desc = "Policy for the TSA to use when creating the time stamp token",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.policy,
+	},
+	{
+		.name = "query",
+		.desc = "Create and print a time stamp request",
+		.type = OPTION_FUNC,
+		.opt.func = ts_opt_query,
+	},
+	{
+		.name = "queryfile",
+		.argname = "file",
+		.desc = "File containing a DER-encoded time stamp request",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.queryfile,
+	},
+	{
+		.name = "reply",
+		.desc = "Create a time stamp response",
+		.type = OPTION_FUNC,
+		.opt.func = ts_opt_reply,
+	},
+	{
+		.name = "section",
+		.argname = "arg",
+		.desc = "TSA section containing the settings for response generation",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.section,
+	},
+	{
+		.name = "signer",
+		.argname = "file",
+		.desc = "Signer certificate file",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.signer,
+	},
+	{
+		.name = "text",
+		.desc = "Output in human-readable text format",
+		.type = OPTION_FLAG,
+		.opt.flag = &ts_config.text,
+	},
+	{
+		.name = "token_in",
+		.desc = "Input is a DER-encoded time stamp token",
+		.type = OPTION_FLAG,
+		.opt.flag = &ts_config.token_in,
+	},
+	{
+		.name = "token_out",
+		.desc = "Output is a DER-encoded time stamp token",
+		.type = OPTION_FLAG,
+		.opt.flag = &ts_config.token_out,
+	},
+	{
+		.name = "untrusted",
+		.argname = "file",
+		.desc = "File containing untrusted certificates",
+		.type = OPTION_ARG,
+		.opt.arg = &ts_config.untrusted,
+	},
+	{
+		.name = "verify",
+		.desc = "Verify a time stamp response",
+		.type = OPTION_FUNC,
+		.opt.func = ts_opt_verify,
+	},
+	{
+		.name = NULL,
+		.desc = "",
+		.type = OPTION_ARGV_FUNC,
+		.opt.argvfunc = ts_opt_md,
+	},
+	{ NULL },
+};
+
+static void
+ts_usage(void)
+{
+	fprintf(stderr, "usage:\n"
+	    "ts -query [-md4 | -md5 | -ripemd160 | -sha1] [-cert]\n"
+	    "    [-config configfile] [-data file_to_hash]\n"
+	    "    [-digest digest_bytes] [-in request.tsq] [-no_nonce]\n"
+	    "    [-out request.tsq] [-policy object_id] [-text]\n");
+	fprintf(stderr, "\n"
+	    "ts -reply [-chain certs_file.pem] [-config configfile]\n"
+	    "    [-in response.tsr] [-inkey private.pem] [-out response.tsr]\n"
+	    "    [-passin arg] [-policy object_id] [-queryfile request.tsq]\n"
+	    "    [-section tsa_section] [-signer tsa_cert.pem] [-text]\n"
+	    "    [-token_in] [-token_out]\n");
+	fprintf(stderr, "\n"
+	    "ts -verify [-CAfile trusted_certs.pem]\n"
+	    "    [-CApath trusted_cert_path] [-data file_to_hash]\n"
+	    "    [-digest digest_bytes] [-in response.tsr]\n"
+	    "    [-queryfile request.tsq] [-token_in]\n"
+	    "    [-untrusted cert_file.pem]\n");
+	fprintf(stderr, "\n");
+	options_usage(ts_options);
+	fprintf(stderr, "\n");
+}
 
 int
 ts_main(int argc, char **argv)
 {
 	int ret = 1;
-	char *configfile = NULL;
-	char *section = NULL;
 	CONF *conf = NULL;
-	enum mode {
-		CMD_NONE, CMD_QUERY, CMD_REPLY, CMD_VERIFY
-	} mode = CMD_NONE;
-	char *data = NULL;
-	char *digest = NULL;
-	const EVP_MD *md = NULL;
-	char *policy = NULL;
-	int no_nonce = 0;
-	int cert = 0;
-	char *in = NULL;
-	char *out = NULL;
-	int text = 0;
-	char *queryfile = NULL;
-	char *passin = NULL;	/* Password source. */
 	char *password = NULL;	/* Password itself. */
-	char *inkey = NULL;
-	char *signer = NULL;
-	char *chain = NULL;
-	char *ca_path = NULL;
-	char *ca_file = NULL;
-	char *untrusted = NULL;
-	/* Input is ContentInfo instead of TimeStampResp. */
-	int token_in = 0;
-	/* Output is ContentInfo instead of TimeStampResp. */
-	int token_out = 0;
 
 	if (single_execution) {
 		if (pledge("stdio cpath wpath rpath tty", NULL) == -1) {
@@ -156,98 +384,15 @@ ts_main(int argc, char **argv)
 		}
 	}
 
-	for (argc--, argv++; argc > 0; argc--, argv++) {
-		if (strcmp(*argv, "-config") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			configfile = *++argv;
-		} else if (strcmp(*argv, "-section") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			section = *++argv;
-		} else if (strcmp(*argv, "-query") == 0) {
-			if (mode != CMD_NONE)
-				goto usage;
-			mode = CMD_QUERY;
-		} else if (strcmp(*argv, "-data") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			data = *++argv;
-		} else if (strcmp(*argv, "-digest") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			digest = *++argv;
-		} else if (strcmp(*argv, "-policy") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			policy = *++argv;
-		} else if (strcmp(*argv, "-no_nonce") == 0) {
-			no_nonce = 1;
-		} else if (strcmp(*argv, "-cert") == 0) {
-			cert = 1;
-		} else if (strcmp(*argv, "-in") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			in = *++argv;
-		} else if (strcmp(*argv, "-token_in") == 0) {
-			token_in = 1;
-		} else if (strcmp(*argv, "-out") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			out = *++argv;
-		} else if (strcmp(*argv, "-token_out") == 0) {
-			token_out = 1;
-		} else if (strcmp(*argv, "-text") == 0) {
-			text = 1;
-		} else if (strcmp(*argv, "-reply") == 0) {
-			if (mode != CMD_NONE)
-				goto usage;
-			mode = CMD_REPLY;
-		} else if (strcmp(*argv, "-queryfile") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			queryfile = *++argv;
-		} else if (strcmp(*argv, "-passin") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			passin = *++argv;
-		} else if (strcmp(*argv, "-inkey") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			inkey = *++argv;
-		} else if (strcmp(*argv, "-signer") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			signer = *++argv;
-		} else if (strcmp(*argv, "-chain") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			chain = *++argv;
-		} else if (strcmp(*argv, "-verify") == 0) {
-			if (mode != CMD_NONE)
-				goto usage;
-			mode = CMD_VERIFY;
-		} else if (strcmp(*argv, "-CApath") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			ca_path = *++argv;
-		} else if (strcmp(*argv, "-CAfile") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			ca_file = *++argv;
-		} else if (strcmp(*argv, "-untrusted") == 0) {
-			if (argc-- < 1)
-				goto usage;
-			untrusted = *++argv;
-		} else if ((md = EVP_get_digestbyname(*argv + 1)) != NULL) {
-			/* empty. */
-		} else
-			goto usage;
-	}
+	memset(&ts_config, 0, sizeof(ts_config));
+	ts_config.mode = CMD_NONE;
+
+	if (options_parse(argc, argv, ts_options, NULL, NULL) != 0)
+		goto usage;
 
 	/* Get the password if required. */
-	if (mode == CMD_REPLY && passin &&
-	    !app_passwd(bio_err, passin, NULL, &password, NULL)) {
+	if (ts_config.mode == CMD_REPLY && ts_config.passin != NULL &&
+	    !app_passwd(bio_err, ts_config.passin, NULL, &password, NULL)) {
 		BIO_printf(bio_err, "Error getting password.\n");
 		goto cleanup;
 	}
@@ -255,7 +400,7 @@ ts_main(int argc, char **argv)
 	 * Check consistency of parameters and execute the appropriate
 	 * function.
 	 */
-	switch (mode) {
+	switch (ts_config.mode) {
 	case CMD_NONE:
 		goto usage;
 	case CMD_QUERY:
@@ -263,64 +408,56 @@ ts_main(int argc, char **argv)
 		 * Data file and message imprint cannot be specified at the
 		 * same time.
 		 */
-		ret = data != NULL && digest != NULL;
+		ret = ts_config.data != NULL && ts_config.digest != NULL;
 		if (ret)
 			goto usage;
 		/* Load the config file for possible policy OIDs. */
-		conf = load_config_file(configfile);
-		ret = !query_command(data, digest, md, policy, no_nonce, cert,
-		    in, out, text);
+		conf = load_config_file(ts_config.configfile);
+		ret = !query_command(ts_config.data, ts_config.digest,
+		    ts_config.md, ts_config.policy, ts_config.no_nonce,
+		    ts_config.cert, ts_config.in, ts_config.out,
+		    ts_config.text);
 		break;
 	case CMD_REPLY:
-		conf = load_config_file(configfile);
-		if (in == NULL) {
-			ret = !(queryfile != NULL && conf != NULL && !token_in);
+		conf = load_config_file(ts_config.configfile);
+		if (ts_config.in == NULL) {
+			ret = !(ts_config.queryfile != NULL && conf != NULL &&
+			    !ts_config.token_in);
 			if (ret)
 				goto usage;
 		} else {
 			/* 'in' and 'queryfile' are exclusive. */
-			ret = !(queryfile == NULL);
+			ret = !(ts_config.queryfile == NULL);
 			if (ret)
 				goto usage;
 		}
 
-		ret = !reply_command(conf, section, queryfile,
-		    password, inkey, signer, chain, policy,
-		    in, token_in, out, token_out, text);
+		ret = !reply_command(conf, ts_config.section,
+		    ts_config.queryfile, password, ts_config.inkey,
+		    ts_config.signer, ts_config.chain, ts_config.policy,
+		    ts_config.in, ts_config.token_in, ts_config.out,
+		    ts_config.token_out, ts_config.text);
 		break;
 	case CMD_VERIFY:
-		ret = !(((queryfile && !data && !digest) ||
-		    (!queryfile && data && !digest) ||
-		    (!queryfile && !data && digest)) && in != NULL);
+		ret = !(((ts_config.queryfile != NULL && ts_config.data == NULL &&
+		    ts_config.digest == NULL) ||
+		    (ts_config.queryfile == NULL && ts_config.data != NULL &&
+		    ts_config.digest == NULL) ||
+		    (ts_config.queryfile == NULL && ts_config.data == NULL &&
+		    ts_config.digest != NULL)) &&
+		    ts_config.in != NULL);
 		if (ret)
 			goto usage;
 
-		ret = !verify_command(data, digest, queryfile, in, token_in,
-		    ca_path, ca_file, untrusted);
+		ret = !verify_command(ts_config.data, ts_config.digest,
+		    ts_config.queryfile, ts_config.in, ts_config.token_in,
+		    ts_config.ca_path, ts_config.ca_file, ts_config.untrusted);
 	}
 
 	goto cleanup;
 
  usage:
-	BIO_printf(bio_err, "usage:\n"
-	    "ts -query [-config configfile] "
-	    "[-data file_to_hash] [-digest digest_bytes]"
-	    "[-md4|-md5|-sha1|-ripemd160] "
-	    "[-policy object_id] [-no_nonce] [-cert] "
-	    "[-in request.tsq] [-out request.tsq] [-text]\n");
-	BIO_printf(bio_err, "or\n"
-	    "ts -reply [-config configfile] [-section tsa_section] "
-	    "[-queryfile request.tsq] [-passin password] "
-	    "[-signer tsa_cert.pem] [-inkey private_key.pem] "
-	    "[-chain certs_file.pem] [-policy object_id] "
-	    "[-in response.tsr] [-token_in] "
-	    "[-out response.tsr] [-token_out] [-text]\n");
-	BIO_printf(bio_err, "or\n"
-	    "ts -verify [-data file_to_hash] [-digest digest_bytes] "
-	    "[-queryfile request.tsq] "
-	    "-in response.tsr [-token_in] "
-	    "-CApath ca_path -CAfile ca_file.pem "
-	    "-untrusted cert_file.pem\n");
+	ts_usage();
 
  cleanup:
 	/* Clean up. */
@@ -340,7 +477,7 @@ txt2obj(const char *oid)
 {
 	ASN1_OBJECT *oid_obj = NULL;
 
-	if (!(oid_obj = OBJ_txt2obj(oid, 0)))
+	if ((oid_obj = OBJ_txt2obj(oid, 0)) == NULL)
 		BIO_printf(bio_err, "cannot convert %s to OID\n", oid);
 
 	return oid_obj;
@@ -352,11 +489,11 @@ load_config_file(const char *configfile)
 	CONF *conf = NULL;
 	long errorline = -1;
 
-	if (!configfile)
+	if (configfile == NULL)
 		configfile = getenv("OPENSSL_CONF");
 
-	if (configfile &&
-	    (!(conf = NCONF_new(NULL)) ||
+	if (configfile != NULL &&
+	    ((conf = NCONF_new(NULL)) == NULL ||
 	    NCONF_load(conf, configfile, &errorline) <= 0)) {
 		if (errorline <= 0)
 			BIO_printf(bio_err, "error loading the config file "
@@ -373,7 +510,7 @@ load_config_file(const char *configfile)
 		p = NCONF_get_string(conf, NULL, ENV_OID_FILE);
 		if (p != NULL) {
 			BIO *oid_bio = BIO_new_file(p, "r");
-			if (!oid_bio)
+			if (oid_bio == NULL)
 				ERR_print_errors(bio_err);
 			else {
 				OBJ_create_objects(oid_bio);
@@ -392,9 +529,9 @@ load_config_file(const char *configfile)
  */
 
 static int
-query_command(const char *data, char *digest, const EVP_MD * md,
-    const char *policy, int no_nonce, int cert, const char *in,
-    const char *out, int text)
+query_command(const char *data, char *digest, const EVP_MD *md,
+    const char *policy, int no_nonce, int cert, const char *in, const char *out,
+    int text)
 {
 	int ret = 0;
 	TS_REQ *query = NULL;
@@ -409,8 +546,8 @@ query_command(const char *data, char *digest, const EVP_MD * md,
 		query = d2i_TS_REQ_bio(in_bio, NULL);
 	} else {
 		/* Open the file if no explicit digest bytes were specified. */
-		if (!digest &&
-		    !(data_bio = BIO_open_with_default(data, "rb", stdin)))
+		if (digest == NULL &&
+		    (data_bio = BIO_open_with_default(data, "rb", stdin)) == NULL)
 			goto end;
 		/* Creating the query object. */
 		query = create_query(data_bio, digest, md,
@@ -448,15 +585,15 @@ query_command(const char *data, char *digest, const EVP_MD * md,
 }
 
 static BIO *
-BIO_open_with_default(const char *file, const char *mode, FILE * default_fp)
+BIO_open_with_default(const char *file, const char *mode, FILE *default_fp)
 {
 	return file == NULL ? BIO_new_fp(default_fp, BIO_NOCLOSE) :
 	    BIO_new_file(file, mode);
 }
 
 static TS_REQ *
-create_query(BIO * data_bio, char *digest, const EVP_MD * md,
-    const char *policy, int no_nonce, int cert)
+create_query(BIO *data_bio, char *digest, const EVP_MD *md, const char *policy,
+    int no_nonce, int cert)
 {
 	int ret = 0;
 	TS_REQ *ts_req = NULL;
@@ -468,11 +605,11 @@ create_query(BIO * data_bio, char *digest, const EVP_MD * md,
 	ASN1_INTEGER *nonce_asn1 = NULL;
 
 	/* Setting default message digest. */
-	if (!md && !(md = EVP_get_digestbyname("sha1")))
+	if (md == NULL && (md = EVP_get_digestbyname("sha1")) == NULL)
 		goto err;
 
 	/* Creating request object. */
-	if (!(ts_req = TS_REQ_new()))
+	if ((ts_req = TS_REQ_new()) == NULL)
 		goto err;
 
 	/* Setting version. */
@@ -480,15 +617,15 @@ create_query(BIO * data_bio, char *digest, const EVP_MD * md,
 		goto err;
 
 	/* Creating and adding MSG_IMPRINT object. */
-	if (!(msg_imprint = TS_MSG_IMPRINT_new()))
+	if ((msg_imprint = TS_MSG_IMPRINT_new()) == NULL)
 		goto err;
 
 	/* Adding algorithm. */
-	if (!(algo = X509_ALGOR_new()))
+	if ((algo = X509_ALGOR_new()) == NULL)
 		goto err;
-	if (!(algo->algorithm = OBJ_nid2obj(EVP_MD_type(md))))
+	if ((algo->algorithm = OBJ_nid2obj(EVP_MD_type(md))) == NULL)
 		goto err;
-	if (!(algo->parameter = ASN1_TYPE_new()))
+	if ((algo->parameter = ASN1_TYPE_new()) == NULL)
 		goto err;
 	algo->parameter->type = V_ASN1_NULL;
 	if (!TS_MSG_IMPRINT_set_algo(msg_imprint, algo))
@@ -504,15 +641,15 @@ create_query(BIO * data_bio, char *digest, const EVP_MD * md,
 		goto err;
 
 	/* Setting policy if requested. */
-	if (policy && !(policy_obj = txt2obj(policy)))
+	if (policy != NULL && (policy_obj = txt2obj(policy)) == NULL)
 		goto err;
-	if (policy_obj && !TS_REQ_set_policy_id(ts_req, policy_obj))
+	if (policy_obj != NULL && !TS_REQ_set_policy_id(ts_req, policy_obj))
 		goto err;
 
 	/* Setting nonce if requested. */
-	if (!no_nonce && !(nonce_asn1 = create_nonce(NONCE_LENGTH)))
+	if (!no_nonce && (nonce_asn1 = create_nonce(NONCE_LENGTH)) == NULL)
 		goto err;
-	if (nonce_asn1 && !TS_REQ_set_nonce(ts_req, nonce_asn1))
+	if (nonce_asn1 != NULL && !TS_REQ_set_nonce(ts_req, nonce_asn1))
 		goto err;
 
 	/* Setting certificate request flag if requested. */
@@ -537,17 +674,18 @@ create_query(BIO * data_bio, char *digest, const EVP_MD * md,
 }
 
 static int
-create_digest(BIO * input, char *digest, const EVP_MD * md,
+create_digest(BIO *input, char *digest, const EVP_MD *md,
     unsigned char **md_value)
 {
 	int md_value_len;
+	EVP_MD_CTX *md_ctx = NULL;
 
 	md_value_len = EVP_MD_size(md);
 	if (md_value_len < 0)
 		goto err;
-	if (input) {
+
+	if (input != NULL) {
 		/* Digest must be computed from an input file. */
-		EVP_MD_CTX *md_ctx;
 		unsigned char buffer[4096];
 		int length;
 
@@ -558,18 +696,26 @@ create_digest(BIO * input, char *digest, const EVP_MD * md,
 		if ((md_ctx = EVP_MD_CTX_new()) == NULL)
 			goto err;
 
-		EVP_DigestInit(md_ctx, md);
+		if (!EVP_DigestInit(md_ctx, md))
+			goto err;
+
 		while ((length = BIO_read(input, buffer, sizeof(buffer))) > 0) {
-			EVP_DigestUpdate(md_ctx, buffer, length);
+			if (!EVP_DigestUpdate(md_ctx, buffer, length))
+				goto err;
 		}
-		EVP_DigestFinal(md_ctx, *md_value, NULL);
+
+		if (!EVP_DigestFinal(md_ctx, *md_value, NULL))
+			goto err;
 
 		EVP_MD_CTX_free(md_ctx);
+		md_ctx = NULL;
+
 	} else {
 		/* Digest bytes are specified with digest. */
 		long digest_len;
+
 		*md_value = string_to_hex(digest, &digest_len);
-		if (!*md_value || md_value_len != digest_len) {
+		if (*md_value == NULL || md_value_len != digest_len) {
 			free(*md_value);
 			*md_value = NULL;
 			BIO_printf(bio_err, "bad digest, %d bytes "
@@ -579,7 +725,9 @@ create_digest(BIO * input, char *digest, const EVP_MD * md,
 	}
 
 	return md_value_len;
+
  err:
+	EVP_MD_CTX_free(md_ctx);
 	return 0;
 }
 
@@ -599,12 +747,12 @@ create_nonce(int bits)
 	/* Find the first non-zero byte and creating ASN1_INTEGER object. */
 	for (i = 0; i < len && !buf[i]; ++i)
 		;
-	if (!(nonce = ASN1_INTEGER_new()))
+	if ((nonce = ASN1_INTEGER_new()) == NULL)
 		goto err;
 	free(nonce->data);
 	/* Allocate at least one byte. */
 	nonce->length = len - i;
-	if (!(nonce->data = malloc(nonce->length + 1)))
+	if ((nonce->data = malloc(nonce->length + 1)) == NULL)
 		goto err;
 	memcpy(nonce->data, buf + i, nonce->length);
 
@@ -615,14 +763,15 @@ create_nonce(int bits)
 	ASN1_INTEGER_free(nonce);
 	return NULL;
 }
+
 /*
  * Reply-related method definitions.
  */
 
 static int
-reply_command(CONF * conf, char *section, char *queryfile,
-    char *passin, char *inkey, char *signer, char *chain, const char *policy,
-    char *in, int token_in, char *out, int token_out, int text)
+reply_command(CONF *conf, char *section, char *queryfile, char *passin,
+    char *inkey, char *signer, char *chain, const char *policy, char *in,
+    int token_in, char *out, int token_out, int text)
 {
 	int ret = 0;
 	TS_RESP *response = NULL;
@@ -647,10 +796,9 @@ reply_command(CONF * conf, char *section, char *queryfile,
 			response = d2i_TS_RESP_bio(in_bio, NULL);
 		}
 	} else {
-		response = create_response(conf, section, queryfile,
-		    passin, inkey, signer, chain,
-		    policy);
-		if (response)
+		response = create_response(conf, section, queryfile, passin,
+				inkey, signer, chain, policy);
+		if (response != NULL)
 			BIO_printf(bio_err, "Response has been generated.\n");
 		else
 			BIO_printf(bio_err, "Response is not generated.\n");
@@ -701,7 +849,7 @@ reply_command(CONF * conf, char *section, char *queryfile,
 
 /* Reads a PKCS7 token and adds default 'granted' status info to it. */
 static TS_RESP *
-read_PKCS7(BIO * in_bio)
+read_PKCS7(BIO *in_bio)
 {
 	int ret = 0;
 	PKCS7 *token = NULL;
@@ -710,19 +858,19 @@ read_PKCS7(BIO * in_bio)
 	TS_STATUS_INFO *si = NULL;
 
 	/* Read PKCS7 object and extract the signed time stamp info. */
-	if (!(token = d2i_PKCS7_bio(in_bio, NULL)))
+	if ((token = d2i_PKCS7_bio(in_bio, NULL)) == NULL)
 		goto end;
-	if (!(tst_info = PKCS7_to_TS_TST_INFO(token)))
+	if ((tst_info = PKCS7_to_TS_TST_INFO(token)) == NULL)
 		goto end;
 
 	/* Creating response object. */
-	if (!(resp = TS_RESP_new()))
+	if ((resp = TS_RESP_new()) == NULL)
 		goto end;
 
 	/* Create granted status info. */
-	if (!(si = TS_STATUS_INFO_new()))
+	if ((si = TS_STATUS_INFO_new()) == NULL)
 		goto end;
-	if (!(ASN1_INTEGER_set(si->status, TS_STATUS_GRANTED)))
+	if (!TS_STATUS_INFO_set_status(si, TS_STATUS_GRANTED))
 		goto end;
 	if (!TS_RESP_set_status_info(resp, si))
 		goto end;
@@ -745,24 +893,23 @@ read_PKCS7(BIO * in_bio)
 }
 
 static TS_RESP *
-create_response(CONF * conf, const char *section,
-    char *queryfile, char *passin, char *inkey,
-    char *signer, char *chain, const char *policy)
+create_response(CONF *conf, const char *section, char *queryfile, char *passin,
+    char *inkey, char *signer, char *chain, const char *policy)
 {
 	int ret = 0;
 	TS_RESP *response = NULL;
 	BIO *query_bio = NULL;
 	TS_RESP_CTX *resp_ctx = NULL;
 
-	if (!(query_bio = BIO_new_file(queryfile, "rb")))
+	if ((query_bio = BIO_new_file(queryfile, "rb")) == NULL)
 		goto end;
 
 	/* Getting TSA configuration section. */
-	if (!(section = TS_CONF_get_tsa_section(conf, section)))
+	if ((section = TS_CONF_get_tsa_section(conf, section)) == NULL)
 		goto end;
 
 	/* Setting up response generation context. */
-	if (!(resp_ctx = TS_RESP_CTX_new()))
+	if ((resp_ctx = TS_RESP_CTX_new()) == NULL)
 		goto end;
 
 	/* Setting serial number provider callback. */
@@ -814,7 +961,7 @@ create_response(CONF * conf, const char *section,
 		goto end;
 
 	/* Creating the response. */
-	if (!(response = TS_RESP_create_response(resp_ctx, query_bio)))
+	if ((response = TS_RESP_create_response(resp_ctx, query_bio)) == NULL)
 		goto end;
 
 	ret = 1;
@@ -830,12 +977,12 @@ create_response(CONF * conf, const char *section,
 }
 
 static ASN1_INTEGER *
-serial_cb(TS_RESP_CTX * ctx, void *data)
+serial_cb(TS_RESP_CTX *ctx, void *data)
 {
 	const char *serial_file = (const char *) data;
 	ASN1_INTEGER *serial = next_serial(serial_file);
 
-	if (!serial) {
+	if (serial == NULL) {
 		TS_RESP_CTX_set_status_info(ctx, TS_STATUS_REJECTION,
 		    "Error during serial number "
 		    "generation.");
@@ -855,10 +1002,10 @@ next_serial(const char *serialfile)
 	ASN1_INTEGER *serial = NULL;
 	BIGNUM *bn = NULL;
 
-	if (!(serial = ASN1_INTEGER_new()))
+	if ((serial = ASN1_INTEGER_new()) == NULL)
 		goto err;
 
-	if (!(in = BIO_new_file(serialfile, "r"))) {
+	if ((in = BIO_new_file(serialfile, "r")) == NULL) {
 		ERR_clear_error();
 		BIO_printf(bio_err, "Warning: could not open file %s for "
 		    "reading, using serial number: 1\n", serialfile);
@@ -871,13 +1018,13 @@ next_serial(const char *serialfile)
 			    serialfile);
 			goto err;
 		}
-		if (!(bn = ASN1_INTEGER_to_BN(serial, NULL)))
+		if ((bn = ASN1_INTEGER_to_BN(serial, NULL)) == NULL)
 			goto err;
 		ASN1_INTEGER_free(serial);
 		serial = NULL;
 		if (!BN_add_word(bn, 1))
 			goto err;
-		if (!(serial = BN_to_ASN1_INTEGER(bn, NULL)))
+		if ((serial = BN_to_ASN1_INTEGER(bn, NULL)) == NULL)
 			goto err;
 	}
 	ret = 1;
@@ -892,12 +1039,12 @@ next_serial(const char *serialfile)
 }
 
 static int
-save_ts_serial(const char *serialfile, ASN1_INTEGER * serial)
+save_ts_serial(const char *serialfile, ASN1_INTEGER *serial)
 {
 	int ret = 0;
 	BIO *out = NULL;
 
-	if (!(out = BIO_new_file(serialfile, "w")))
+	if ((out = BIO_new_file(serialfile, "w")) == NULL)
 		goto err;
 	if (i2a_ASN1_INTEGER(out, serial) <= 0)
 		goto err;
@@ -927,18 +1074,18 @@ verify_command(char *data, char *digest, char *queryfile, char *in,
 	int ret = 0;
 
 	/* Decode the token (PKCS7) or response (TS_RESP) files. */
-	if (!(in_bio = BIO_new_file(in, "rb")))
+	if ((in_bio = BIO_new_file(in, "rb")) == NULL)
 		goto end;
 	if (token_in) {
-		if (!(token = d2i_PKCS7_bio(in_bio, NULL)))
+		if ((token = d2i_PKCS7_bio(in_bio, NULL)) == NULL)
 			goto end;
 	} else {
-		if (!(response = d2i_TS_RESP_bio(in_bio, NULL)))
+		if ((response = d2i_TS_RESP_bio(in_bio, NULL)) == NULL)
 			goto end;
 	}
 
-	if (!(verify_ctx = create_verify_ctx(data, digest, queryfile,
-	    ca_path, ca_file, untrusted)))
+	if ((verify_ctx = create_verify_ctx(data, digest, queryfile,
+	    ca_path, ca_file, untrusted)) == NULL)
 		goto end;
 
 	/* Checking the token or response against the request. */
@@ -971,50 +1118,61 @@ create_verify_ctx(char *data, char *digest, char *queryfile, char *ca_path,
 	TS_VERIFY_CTX *ctx = NULL;
 	BIO *input = NULL;
 	TS_REQ *request = NULL;
+	X509_STORE *store;
+	STACK_OF(X509) *certs;
 	int ret = 0;
 
 	if (data != NULL || digest != NULL) {
-		if (!(ctx = TS_VERIFY_CTX_new()))
+		if ((ctx = TS_VERIFY_CTX_new()) == NULL)
 			goto err;
-		ctx->flags = TS_VFY_VERSION | TS_VFY_SIGNER;
+		TS_VERIFY_CTX_set_flags(ctx, TS_VFY_VERSION | TS_VFY_SIGNER);
 		if (data != NULL) {
-			ctx->flags |= TS_VFY_DATA;
-			if (!(ctx->data = BIO_new_file(data, "rb")))
+			BIO *data_bio;
+
+			TS_VERIFY_CTX_add_flags(ctx, TS_VFY_DATA);
+			if ((data_bio = BIO_new_file(data, "rb")) == NULL)
 				goto err;
+			TS_VERIFY_CTX_set_data(ctx, data_bio);
 		} else if (digest != NULL) {
+			unsigned char *imprint;
 			long imprint_len;
-			ctx->flags |= TS_VFY_IMPRINT;
-			if (!(ctx->imprint = string_to_hex(digest,
-				    &imprint_len))) {
+
+			TS_VERIFY_CTX_add_flags(ctx, TS_VFY_IMPRINT);
+			if ((imprint = string_to_hex(digest,
+			    &imprint_len)) == NULL) {
 				BIO_printf(bio_err, "invalid digest string\n");
 				goto err;
 			}
-			ctx->imprint_len = imprint_len;
+			TS_VERIFY_CTX_set_imprint(ctx, imprint, imprint_len);
 		}
 	} else if (queryfile != NULL) {
 		/*
 		 * The request has just to be read, decoded and converted to
 		 * a verify context object.
 		 */
-		if (!(input = BIO_new_file(queryfile, "rb")))
+		if ((input = BIO_new_file(queryfile, "rb")) == NULL)
 			goto err;
-		if (!(request = d2i_TS_REQ_bio(input, NULL)))
+		if ((request = d2i_TS_REQ_bio(input, NULL)) == NULL)
 			goto err;
-		if (!(ctx = TS_REQ_to_TS_VERIFY_CTX(request, NULL)))
+		if ((ctx = TS_REQ_to_TS_VERIFY_CTX(request, NULL)) == NULL)
 			goto err;
 	} else
 		return NULL;
 
 	/* Add the signature verification flag and arguments. */
-	ctx->flags |= TS_VFY_SIGNATURE;
+	TS_VERIFY_CTX_add_flags(ctx, TS_VFY_SIGNATURE);
 
 	/* Initialising the X509_STORE object. */
-	if (!(ctx->store = create_cert_store(ca_path, ca_file)))
+	if ((store = create_cert_store(ca_path, ca_file)) == NULL)
 		goto err;
+	TS_VERIFY_CTX_set_store(ctx, store);
 
 	/* Loading untrusted certificates. */
-	if (untrusted && !(ctx->certs = TS_CONF_load_certs(untrusted)))
-		goto err;
+	if (untrusted != NULL) {
+		if ((certs = TS_CONF_load_certs(untrusted)) == NULL)
+			goto err;
+		TS_VERIFY_CTX_set_certs(ctx, certs);
+	}
 
 	ret = 1;
  err:
@@ -1035,13 +1193,14 @@ create_cert_store(char *ca_path, char *ca_file)
 	int i;
 
 	/* Creating the X509_STORE object. */
-	cert_ctx = X509_STORE_new();
+	if ((cert_ctx = X509_STORE_new()) == NULL)
+		goto err;
 
 	/* Setting the callback for certificate chain verification. */
 	X509_STORE_set_verify_cb(cert_ctx, verify_cb);
 
 	/* Adding a trusted certificate directory source. */
-	if (ca_path) {
+	if (ca_path != NULL) {
 		lookup = X509_STORE_add_lookup(cert_ctx,
 		    X509_LOOKUP_hash_dir());
 		if (lookup == NULL) {
@@ -1056,7 +1215,7 @@ create_cert_store(char *ca_path, char *ca_file)
 		}
 	}
 	/* Adding a trusted certificate file source. */
-	if (ca_file) {
+	if (ca_file != NULL) {
 		lookup = X509_STORE_add_lookup(cert_ctx, X509_LOOKUP_file());
 		if (lookup == NULL) {
 			BIO_printf(bio_err, "memory allocation failure\n");
@@ -1075,7 +1234,7 @@ create_cert_store(char *ca_path, char *ca_file)
 }
 
 static int
-verify_cb(int ok, X509_STORE_CTX * ctx)
+verify_cb(int ok, X509_STORE_CTX *ctx)
 {
 	/*
 	char buf[256];
