@@ -1,4 +1,4 @@
-/* $OpenBSD: asn1_gen.c,v 1.17 2018/04/25 11:48:21 tb Exp $ */
+/* $OpenBSD: asn1_gen.c,v 1.19 2022/05/24 19:56:13 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2002.
  */
@@ -62,6 +62,8 @@
 #include <openssl/err.h>
 #include <openssl/x509v3.h>
 
+#include "asn1_locl.h"
+
 #define ASN1_GEN_FLAG		0x10000
 #define ASN1_GEN_FLAG_IMP	(ASN1_GEN_FLAG|1)
 #define ASN1_GEN_FLAG_EXP	(ASN1_GEN_FLAG|2)
@@ -258,7 +260,7 @@ ASN1_generate_v3(const char *str, X509V3_CTX *cnf)
 	/* Obtain new ASN1_TYPE structure */
 	ret = d2i_ASN1_TYPE(NULL, &cp, len);
 
-err:
+ err:
 	free(orig_der);
 	free(new_der);
 
@@ -478,7 +480,7 @@ asn1_multi(int utype, const char *section, X509V3_CTX *cnf)
 
 	der = NULL;
 
-bad:
+ bad:
 	free(der);
 	if (sk)
 		sk_ASN1_TYPE_pop_free(sk, ASN1_TYPE_free);
@@ -754,10 +756,9 @@ asn1_str2type(const char *str, int format, int utype)
 		}
 
 		if ((utype == V_ASN1_BIT_STRING) && no_unused) {
-			atmp->value.asn1_string->flags &=
-			    ~(ASN1_STRING_FLAG_BITS_LEFT | 0x07);
-			atmp->value.asn1_string->flags |=
-			    ASN1_STRING_FLAG_BITS_LEFT;
+			if (!asn1_abs_set_unused_bits(atmp->value.asn1_string,
+			    0))
+				goto bad_str;
 		}
 
 		break;
@@ -771,9 +772,9 @@ asn1_str2type(const char *str, int format, int utype)
 	atmp->type = utype;
 	return atmp;
 
-bad_str:
+ bad_str:
 	ERR_asprintf_error_data("string=%s", str);
-bad_form:
+ bad_form:
 	ASN1_TYPE_free(atmp);
 	return NULL;
 }
