@@ -1,25 +1,25 @@
-/* $OpenBSD: dsa_lib.c,v 1.28 2018/02/20 17:52:27 tb Exp $ */
+/* $OpenBSD: dsa_lib.c,v 1.37 2022/08/31 13:28:39 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
  * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
- * 
+ *
  * This library is free for commercial and non-commercial use as long as
  * the following conditions are aheared to.  The following conditions
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
  * except that the holder is Tim Hudson (tjh@cryptsoft.com).
- * 
+ *
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
  * If this package is used in a product, Eric Young should be given attribution
  * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -34,10 +34,10 @@
  *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
- * 4. If you include any Windows specific code (or a derivative thereof) from 
+ * 4. If you include any Windows specific code (or a derivative thereof) from
  *    the apps directory (application code) you must include an acknowledgement:
  *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -49,7 +49,7 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- * 
+ *
  * The licence and distribution terms for any publically available version or
  * derivative of this code cannot be changed.  i.e. this code cannot simply be
  * copied and put under another distribution licence
@@ -74,6 +74,9 @@
 #include <openssl/engine.h>
 #endif
 
+#include "dh_local.h"
+#include "dsa_locl.h"
+
 static const DSA_METHOD *default_DSA_method = NULL;
 
 void
@@ -108,10 +111,8 @@ DSA_set_method(DSA *dsa, const DSA_METHOD *meth)
         if (mtmp->finish)
 		mtmp->finish(dsa);
 #ifndef OPENSSL_NO_ENGINE
-	if (dsa->engine) {
-		ENGINE_finish(dsa->engine);
-		dsa->engine = NULL;
-	}
+	ENGINE_finish(dsa->engine);
+	dsa->engine = NULL;
 #endif
         dsa->meth = meth;
         if (meth->init)
@@ -142,7 +143,7 @@ DSA_new_method(ENGINE *engine)
 		ret->engine = ENGINE_get_default_DSA();
 	if (ret->engine) {
 		ret->meth = ENGINE_get_DSA(ret->engine);
-		if (!ret->meth) {
+		if (ret->meth == NULL) {
 			DSAerror(ERR_R_ENGINE_LIB);
 			ENGINE_finish(ret->engine);
 			free(ret);
@@ -153,7 +154,6 @@ DSA_new_method(ENGINE *engine)
 
 	ret->pad = 0;
 	ret->version = 0;
-	ret->write_params = 1;
 	ret->p = NULL;
 	ret->q = NULL;
 	ret->g = NULL;
@@ -170,14 +170,13 @@ DSA_new_method(ENGINE *engine)
 	CRYPTO_new_ex_data(CRYPTO_EX_INDEX_DSA, ret, &ret->ex_data);
 	if (ret->meth->init != NULL && !ret->meth->init(ret)) {
 #ifndef OPENSSL_NO_ENGINE
-		if (ret->engine)
-			ENGINE_finish(ret->engine);
+		ENGINE_finish(ret->engine);
 #endif
 		CRYPTO_free_ex_data(CRYPTO_EX_INDEX_DSA, ret, &ret->ex_data);
 		free(ret);
 		ret = NULL;
 	}
-	
+
 	return ret;
 }
 
@@ -196,8 +195,7 @@ DSA_free(DSA *r)
 	if (r->meth->finish)
 		r->meth->finish(r);
 #ifndef OPENSSL_NO_ENGINE
-	if (r->engine)
-		ENGINE_finish(r->engine);
+	ENGINE_finish(r->engine);
 #endif
 
 	CRYPTO_free_ex_data(CRYPTO_EX_INDEX_DSA, r, &r->ex_data);
@@ -222,23 +220,15 @@ DSA_up_ref(DSA *r)
 int
 DSA_size(const DSA *r)
 {
-	int ret, i;
-	ASN1_INTEGER bs;
-	unsigned char buf[4];	/* 4 bytes looks really small.
-				   However, i2d_ASN1_INTEGER() will not look
-				   beyond the first byte, as long as the second
-				   parameter is NULL. */
+	DSA_SIG signature;
+	int ret = 0;
 
-	i = BN_num_bits(r->q);
-	bs.length = (i + 7) / 8;
-	bs.data = buf;
-	bs.type = V_ASN1_INTEGER;
-	/* If the top bit is set the asn1 encoding is 1 larger. */
-	buf[0] = 0xff;
+	signature.r = r->q;
+	signature.s = r->q;
+
+	if ((ret = i2d_DSA_SIG(&signature, NULL)) < 0)
+		ret = 0;
 
-	i = i2d_ASN1_INTEGER(&bs, NULL);
-	i += i; /* r and s */
-	ret = ASN1_object_size(1, i, V_ASN1_SEQUENCE);
 	return ret;
 }
 
@@ -262,6 +252,15 @@ DSA_get_ex_data(DSA *d, int idx)
 	return CRYPTO_get_ex_data(&d->ex_data, idx);
 }
 
+int
+DSA_security_bits(const DSA *d)
+{
+	if (d->p == NULL || d->q == NULL)
+		return -1;
+
+	return BN_security_bits(BN_num_bits(d->p), BN_num_bits(d->q));
+}
+
 #ifndef OPENSSL_NO_DH
 DH *
 DSA_dup_DH(const DSA *r)
@@ -270,7 +269,7 @@ DSA_dup_DH(const DSA *r)
 	 * DSA has p, q, g, optional pub_key, optional priv_key.
 	 * DH has p, optional length, g, optional pub_key, optional priv_key,
 	 * optional q.
-	 */ 
+	 */
 	DH *ret = NULL;
 
 	if (r == NULL)
@@ -278,7 +277,7 @@ DSA_dup_DH(const DSA *r)
 	ret = DH_new();
 	if (ret == NULL)
 		goto err;
-	if (r->p != NULL) 
+	if (r->p != NULL)
 		if ((ret->p = BN_dup(r->p)) == NULL)
 			goto err;
 	if (r->q != NULL) {
@@ -365,6 +364,36 @@ DSA_set0_key(DSA *d, BIGNUM *pub_key, BIGNUM *priv_key)
 	return 1;
 }
 
+const BIGNUM *
+DSA_get0_p(const DSA *d)
+{
+	return d->p;
+}
+
+const BIGNUM *
+DSA_get0_q(const DSA *d)
+{
+	return d->q;
+}
+
+const BIGNUM *
+DSA_get0_g(const DSA *d)
+{
+	return d->g;
+}
+
+const BIGNUM *
+DSA_get0_pub_key(const DSA *d)
+{
+	return d->pub_key;
+}
+
+const BIGNUM *
+DSA_get0_priv_key(const DSA *d)
+{
+	return d->priv_key;
+}
+
 void
 DSA_clear_flags(DSA *d, int flags)
 {
@@ -388,3 +417,9 @@ DSA_get0_engine(DSA *d)
 {
 	return d->engine;
 }
+
+int
+DSA_bits(const DSA *dsa)
+{
+	return BN_num_bits(dsa->p);
+}
