@@ -1,4 +1,4 @@
-/* $OpenBSD: ec_asn1.c,v 1.35 2022/01/14 08:16:13 tb Exp $ */
+/* $OpenBSD: ec_asn1.c,v 1.37 2022/05/24 20:06:32 tb Exp $ */
 /*
  * Written by Nils Larsch for the OpenSSL project.
  */
@@ -60,11 +60,13 @@
 
 #include <openssl/opensslconf.h>
 
-#include "ec_lcl.h"
 #include <openssl/err.h>
 #include <openssl/asn1t.h>
 #include <openssl/objects.h>
 
+#include "asn1_locl.h"
+#include "ec_lcl.h"
+
 int 
 EC_GROUP_get_basis_type(const EC_GROUP * group)
 {
@@ -860,24 +862,24 @@ ec_asn1_group2curve(const EC_GROUP * group, X9_62_CURVE * curve)
 		ECerror(ERR_R_ASN1_LIB);
 		goto err;
 	}
+
+	ASN1_BIT_STRING_free(curve->seed);
+	curve->seed = NULL;
+
 	/* set the seed (optional) */
-	if (group->seed) {
-		if (!curve->seed)
-			if ((curve->seed = ASN1_BIT_STRING_new()) == NULL) {
-				ECerror(ERR_R_MALLOC_FAILURE);
-				goto err;
-			}
-		curve->seed->flags &= ~(ASN1_STRING_FLAG_BITS_LEFT | 0x07);
-		curve->seed->flags |= ASN1_STRING_FLAG_BITS_LEFT;
+	if (group->seed != NULL) {
+		if ((curve->seed = ASN1_BIT_STRING_new()) == NULL) {
+			ECerror(ERR_R_MALLOC_FAILURE);
+			goto err;
+		}
 		if (!ASN1_BIT_STRING_set(curve->seed, group->seed,
 			(int) group->seed_len)) {
 			ECerror(ERR_R_ASN1_LIB);
 			goto err;
 		}
-	} else {
-		if (curve->seed) {
-			ASN1_BIT_STRING_free(curve->seed);
-			curve->seed = NULL;
+		if (!asn1_abs_set_unused_bits(curve->seed, 0)) {
+			ECerror(ERR_R_ASN1_LIB);
+			goto err;
 		}
 	}
 
@@ -1285,7 +1287,7 @@ EC_GROUP *
 d2i_ECPKParameters(EC_GROUP ** a, const unsigned char **in, long len)
 {
 	EC_GROUP *group = NULL;
-	ECPKPARAMETERS *params = NULL;
+	ECPKPARAMETERS *params;
 
 	if ((params = d2i_ECPKPARAMETERS(NULL, in, len)) == NULL) {
 		ECerror(EC_R_D2I_ECPKPARAMETERS_FAILURE);
@@ -1332,13 +1334,8 @@ d2i_ECPrivateKey(EC_KEY ** a, const unsigned char **in, long len)
 	EC_KEY *ret = NULL;
 	EC_PRIVATEKEY *priv_key = NULL;
 
-	if ((priv_key = EC_PRIVATEKEY_new()) == NULL) {
-		ECerror(ERR_R_MALLOC_FAILURE);
-		return NULL;
-	}
-	if ((priv_key = d2i_EC_PRIVATEKEY(&priv_key, in, len)) == NULL) {
+	if ((priv_key = d2i_EC_PRIVATEKEY(NULL, in, len)) == NULL) {
 		ECerror(ERR_R_EC_LIB);
-		EC_PRIVATEKEY_free(priv_key);
 		return NULL;
 	}
 	if (a == NULL || *a == NULL) {
@@ -1486,10 +1483,11 @@ i2d_ECPrivateKey(EC_KEY * a, unsigned char **out)
 			ECerror(ERR_R_EC_LIB);
 			goto err;
 		}
-		priv_key->publicKey->flags &= ~(ASN1_STRING_FLAG_BITS_LEFT | 0x07);
-		priv_key->publicKey->flags |= ASN1_STRING_FLAG_BITS_LEFT;
-		if (!ASN1_STRING_set(priv_key->publicKey, buffer,
-			buf_len)) {
+		if (!ASN1_STRING_set(priv_key->publicKey, buffer, buf_len)) {
+			ECerror(ERR_R_ASN1_LIB);
+			goto err;
+		}
+		if (!asn1_abs_set_unused_bits(priv_key->publicKey, 0)) {
 			ECerror(ERR_R_ASN1_LIB);
 			goto err;
 		}
