@@ -1,4 +1,4 @@
-/* $OpenBSD: ecs_sign.c,v 1.7 2019/01/19 01:07:00 tb Exp $ */
+/* $OpenBSD: ecs_sign.c,v 1.9 2022/01/27 20:30:29 tb Exp $ */
 /* ====================================================================
  * Copyright (c) 1998-2002 The OpenSSL Project.  All rights reserved.
  *
@@ -58,7 +58,10 @@
 #ifndef OPENSSL_NO_ENGINE
 #include <openssl/engine.h>
 #endif
+#include <openssl/err.h>
+#include <openssl/evp.h>
 
+#include "bn_lcl.h"
 #include "ecs_locl.h"
 #include "ec_lcl.h"
 
