@@ -1,4 +1,4 @@
-/* $OpenBSD: pk7_lib.c,v 1.19 2017/01/29 17:49:23 beck Exp $ */
+/* $OpenBSD: pk7_lib.c,v 1.23 2021/12/12 21:30:14 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -63,6 +63,8 @@
 #include <openssl/x509.h>
 
 #include "asn1_locl.h"
+#include "evp_locl.h"
+#include "x509_lcl.h"
 
 long
 PKCS7_ctrl(PKCS7 *p7, int cmd, long larg, char *parg)
@@ -186,7 +188,6 @@ PKCS7_set_type(PKCS7 *p7, int type)
 		if ((p7->d.signed_and_enveloped =
 		    PKCS7_SIGN_ENVELOPE_new()) == NULL)
 			goto err;
-		ASN1_INTEGER_set(p7->d.signed_and_enveloped->version, 1);
 		if (!ASN1_INTEGER_set(p7->d.signed_and_enveloped->version, 1))
 			goto err;
 		p7->d.signed_and_enveloped->enc_data->content_type =
@@ -374,7 +375,7 @@ PKCS7_SIGNER_INFO_set(PKCS7_SIGNER_INFO *p7i, X509 *x509, EVP_PKEY *pkey,
 	 * things the ugly way. */
 	ASN1_INTEGER_free(p7i->issuer_and_serial->serial);
 	if (!(p7i->issuer_and_serial->serial =
-	    ASN1_STRING_dup(X509_get_serialNumber(x509))))
+	    ASN1_INTEGER_dup(X509_get_serialNumber(x509))))
 		goto err;
 
 	/* lets keep the pkey around for a while */
@@ -534,7 +535,7 @@ PKCS7_RECIP_INFO_set(PKCS7_RECIP_INFO *p7i, X509 *x509)
 
 	ASN1_INTEGER_free(p7i->issuer_and_serial->serial);
 	if (!(p7i->issuer_and_serial->serial =
-	    ASN1_STRING_dup(X509_get_serialNumber(x509))))
+	    ASN1_INTEGER_dup(X509_get_serialNumber(x509))))
 		return 0;
 
 	pkey = X509_get_pubkey(x509);
