@@ -1,4 +1,4 @@
-/* $OpenBSD: x509name.c,v 1.15 2018/03/17 15:28:27 tb Exp $ */
+/* $OpenBSD: x509name.c,v 1.27 2021/11/01 20:53:08 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -66,6 +66,8 @@
 #include <openssl/stack.h>
 #include <openssl/x509.h>
 
+#include "x509_lcl.h"
+
 int
 X509_NAME_get_text_by_NID(X509_NAME *name, int nid, char *buf, int len)
 {
@@ -78,7 +80,7 @@ X509_NAME_get_text_by_NID(X509_NAME *name, int nid, char *buf, int len)
 }
 
 int
-X509_NAME_get_text_by_OBJ(X509_NAME *name, ASN1_OBJECT *obj, char *buf,
+X509_NAME_get_text_by_OBJ(X509_NAME *name, const ASN1_OBJECT *obj, char *buf,
     int len)
 {
 	int i;
@@ -99,7 +101,7 @@ X509_NAME_get_text_by_OBJ(X509_NAME *name, ASN1_OBJECT *obj, char *buf,
 }
 
 int
-X509_NAME_entry_count(X509_NAME *name)
+X509_NAME_entry_count(const X509_NAME *name)
 {
 	if (name == NULL)
 		return (0);
@@ -107,7 +109,7 @@ X509_NAME_entry_count(X509_NAME *name)
 }
 
 int
-X509_NAME_get_index_by_NID(X509_NAME *name, int nid, int lastpos)
+X509_NAME_get_index_by_NID(const X509_NAME *name, int nid, int lastpos)
 {
 	ASN1_OBJECT *obj;
 
@@ -119,7 +121,8 @@ X509_NAME_get_index_by_NID(X509_NAME *name, int nid, int lastpos)
 
 /* NOTE: you should be passsing -1, not 0 as lastpos */
 int
-X509_NAME_get_index_by_OBJ(X509_NAME *name, ASN1_OBJECT *obj, int lastpos)
+X509_NAME_get_index_by_OBJ(const X509_NAME *name, const ASN1_OBJECT *obj,
+    int lastpos)
 {
 	int n;
 	X509_NAME_ENTRY *ne;
@@ -140,7 +143,7 @@ X509_NAME_get_index_by_OBJ(X509_NAME *name, ASN1_OBJECT *obj, int lastpos)
 }
 
 X509_NAME_ENTRY *
-X509_NAME_get_entry(X509_NAME *name, int loc)
+X509_NAME_get_entry(const X509_NAME *name, int loc)
 {
 	if (name == NULL || sk_X509_NAME_ENTRY_num(name->entries) <= loc ||
 	    loc < 0)
@@ -188,8 +191,8 @@ X509_NAME_delete_entry(X509_NAME *name, int loc)
 }
 
 int
-X509_NAME_add_entry_by_OBJ(X509_NAME *name, ASN1_OBJECT *obj, int type,
-    unsigned char *bytes, int len, int loc, int set)
+X509_NAME_add_entry_by_OBJ(X509_NAME *name, const ASN1_OBJECT *obj, int type,
+    const unsigned char *bytes, int len, int loc, int set)
 {
 	X509_NAME_ENTRY *ne;
 	int ret;
@@ -204,7 +207,7 @@ X509_NAME_add_entry_by_OBJ(X509_NAME *name, ASN1_OBJECT *obj, int type,
 
 int
 X509_NAME_add_entry_by_NID(X509_NAME *name, int nid, int type,
-    unsigned char *bytes, int len, int loc, int set)
+    const unsigned char *bytes, int len, int loc, int set)
 {
 	X509_NAME_ENTRY *ne;
 	int ret;
@@ -235,7 +238,8 @@ X509_NAME_add_entry_by_txt(X509_NAME *name, const char *field, int type,
 /* if set is -1, append to previous set, 0 'a new one', and 1,
  * prepend to the guy we are about to stomp on. */
 int
-X509_NAME_add_entry(X509_NAME *name, X509_NAME_ENTRY *ne, int loc, int set)
+X509_NAME_add_entry(X509_NAME *name, const X509_NAME_ENTRY *ne, int loc,
+    int set)
 {
 	X509_NAME_ENTRY *new_name = NULL;
 	int n, i, inc;
@@ -249,17 +253,15 @@ X509_NAME_add_entry(X509_NAME *name, X509_NAME_ENTRY *ne, int loc, int set)
 		loc = n;
 	else if (loc < 0)
 		loc = n;
-
+	inc = (set == 0);
 	name->modified = 1;
 
 	if (set == -1) {
 		if (loc == 0) {
 			set = 0;
 			inc = 1;
-		} else {
+		} else
 			set = sk_X509_NAME_ENTRY_value(sk, loc - 1)->set;
-			inc = 0;
-		}
 	} else /* if (set >= 0) */ {
 		if (loc >= n) {
 			if (loc != 0)
@@ -268,10 +270,10 @@ X509_NAME_add_entry(X509_NAME *name, X509_NAME_ENTRY *ne, int loc, int set)
 				set = 0;
 		} else
 			set = sk_X509_NAME_ENTRY_value(sk, loc)->set;
-		inc = (set == 0) ? 1 : 0;
 	}
 
-	if ((new_name = X509_NAME_ENTRY_dup(ne)) == NULL)
+	/* OpenSSL has ASN1-generated X509_NAME_ENTRY_dup() without const. */
+	if ((new_name = X509_NAME_ENTRY_dup((X509_NAME_ENTRY *)ne)) == NULL)
 		goto err;
 	new_name->set = set;
 	if (!sk_X509_NAME_ENTRY_insert(sk, new_name, loc)) {
@@ -281,7 +283,7 @@ X509_NAME_add_entry(X509_NAME *name, X509_NAME_ENTRY *ne, int loc, int set)
 	if (inc) {
 		n = sk_X509_NAME_ENTRY_num(sk);
 		for (i = loc + 1; i < n; i++)
-			sk_X509_NAME_ENTRY_value(sk, i - 1)->set += 1;
+			sk_X509_NAME_ENTRY_value(sk, i)->set += 1;
 	}
 	return (1);
 
@@ -311,7 +313,7 @@ X509_NAME_ENTRY_create_by_txt(X509_NAME_ENTRY **ne,
 
 X509_NAME_ENTRY *
 X509_NAME_ENTRY_create_by_NID(X509_NAME_ENTRY **ne, int nid, int type,
-    unsigned char *bytes, int len)
+    const unsigned char *bytes, int len)
 {
 	ASN1_OBJECT *obj;
 	X509_NAME_ENTRY *nentry;
@@ -327,8 +329,8 @@ X509_NAME_ENTRY_create_by_NID(X509_NAME_ENTRY **ne, int nid, int type,
 }
 
 X509_NAME_ENTRY *
-X509_NAME_ENTRY_create_by_OBJ(X509_NAME_ENTRY **ne, ASN1_OBJECT *obj, int type,
-    const unsigned char *bytes, int len)
+X509_NAME_ENTRY_create_by_OBJ(X509_NAME_ENTRY **ne, const ASN1_OBJECT *obj,
+    int type, const unsigned char *bytes, int len)
 {
 	X509_NAME_ENTRY *ret;
 
@@ -354,7 +356,7 @@ err:
 }
 
 int
-X509_NAME_ENTRY_set_object(X509_NAME_ENTRY *ne, ASN1_OBJECT *obj)
+X509_NAME_ENTRY_set_object(X509_NAME_ENTRY *ne, const ASN1_OBJECT *obj)
 {
 	if ((ne == NULL) || (obj == NULL)) {
 		X509error(ERR_R_PASSED_NULL_PARAMETER);
@@ -391,7 +393,7 @@ X509_NAME_ENTRY_set_data(X509_NAME_ENTRY *ne, int type,
 }
 
 ASN1_OBJECT *
-X509_NAME_ENTRY_get_object(X509_NAME_ENTRY *ne)
+X509_NAME_ENTRY_get_object(const X509_NAME_ENTRY *ne)
 {
 	if (ne == NULL)
 		return (NULL);
@@ -399,7 +401,7 @@ X509_NAME_ENTRY_get_object(X509_NAME_ENTRY *ne)
 }
 
 ASN1_STRING *
-X509_NAME_ENTRY_get_data(X509_NAME_ENTRY *ne)
+X509_NAME_ENTRY_get_data(const X509_NAME_ENTRY *ne)
 {
 	if (ne == NULL)
 		return (NULL);
