@@ -1,4 +1,4 @@
-/*	$OpenBSD: ssltest.c,v 1.33 2021/11/21 21:40:45 tb Exp $ */
+/*	$OpenBSD: ssltest.c,v 1.35 2022/07/07 13:10:22 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -434,6 +434,7 @@ main(int argc, char *argv[])
 	const SSL_METHOD *meth = NULL;
 	SSL *c_ssl, *s_ssl;
 	int number = 1, reuse = 0;
+	int seclevel = 0;
 	long bytes = 256L;
 	DH *dh;
 	int dhe1024dsa = 0;
@@ -494,6 +495,10 @@ main(int argc, char *argv[])
 			number = atoi(*(++argv));
 			if (number == 0)
 				number = 1;
+		} else if (strncmp(*argv, "-seclevel", 9) == 0) {
+			if (--argc < 1)
+				goto bad;
+			seclevel = atoi(*(++argv));
 		} else if (strcmp(*argv, "-bytes") == 0) {
 			if (--argc < 1)
 				goto bad;
@@ -620,6 +625,9 @@ bad:
 		goto end;
 	}
 
+	SSL_CTX_set_security_level(c_ctx, seclevel);
+	SSL_CTX_set_security_level(s_ctx, seclevel);
+
 	if (cipher != NULL) {
 		SSL_CTX_set_cipher_list(c_ctx, cipher);
 		SSL_CTX_set_cipher_list(s_ctx, cipher);
@@ -659,8 +667,7 @@ bad:
 		EC_KEY_free(ecdh);
 	}
 
-	if (!SSL_CTX_use_certificate_file(s_ctx, server_cert,
-	    SSL_FILETYPE_PEM)) {
+	if (!SSL_CTX_use_certificate_chain_file(s_ctx, server_cert)) {
 		ERR_print_errors(bio_err);
 	} else if (!SSL_CTX_use_PrivateKey_file(s_ctx,
 	    (server_key ? server_key : server_cert), SSL_FILETYPE_PEM)) {
@@ -669,8 +676,7 @@ bad:
 	}
 
 	if (client_auth) {
-		SSL_CTX_use_certificate_file(c_ctx, client_cert,
-		    SSL_FILETYPE_PEM);
+		SSL_CTX_use_certificate_chain_file(c_ctx, client_cert);
 		SSL_CTX_use_PrivateKey_file(c_ctx,
 		    (client_key ? client_key : client_cert),
 		    SSL_FILETYPE_PEM);
